import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:mcomp_flutter_app/helpers/constants.dart';

class CustomAppTheme {
  static ThemeData getLightTheme({required VisualDensity visualDensity}) {
    final colorScheme = _getLightThemeColorScheme();

    return ThemeData(
      brightness: Brightness.light,
      visualDensity: visualDensity,
      useMaterial3: true,
      colorScheme: colorScheme,
      scaffoldBackgroundColor: colorScheme.background,
      appBarTheme: _getAppBarTheme(colorScheme),
      chipTheme: _getChipThemeData(colorScheme),
      iconTheme: _getIconThemeData(colorScheme),
      textTheme: _getTextTheme(colorScheme),
      filledButtonTheme: _getFilledButtonThemeData(colorScheme),
    );
  }

  static ThemeData getDarkTheme({required VisualDensity visualDensity}) {
    final colorScheme = _getDarkThemeColorScheme();

    return ThemeData(
      brightness: Brightness.dark,
      visualDensity: visualDensity,
      useMaterial3: true,
      colorScheme: colorScheme,
      scaffoldBackgroundColor: colorScheme.background,
      appBarTheme: _getAppBarTheme(colorScheme),
      chipTheme: _getChipThemeData(colorScheme),
      iconTheme: _getIconThemeData(colorScheme),
      textTheme: _getTextTheme(colorScheme),
      filledButtonTheme: _getFilledButtonThemeData(colorScheme),
    );
  }

  static ColorScheme _getLightThemeColorScheme() {
    return const ColorScheme(
      brightness: Brightness.light,
      primary: ConstColors.light,
      onPrimary: ConstColors.dark,
      secondary: ConstColors.lightSec,
      onSecondary: ConstColors.dark,
      error: ConstColors.red,
      onError: ConstColors.red,
      background: ConstColors.light,
      onBackground: ConstColors.dark,
      surface: ConstColors.light,
      onSurface: ConstColors.dark,
    );
  }

  static ColorScheme _getDarkThemeColorScheme() {
    return const ColorScheme(
      brightness: Brightness.dark,
      primary: ConstColors.dark,
      onPrimary: ConstColors.light,
      secondary: ConstColors.darkSec,
      onSecondary: ConstColors.light,
      error: ConstColors.red,
      onError: ConstColors.red,
      background: ConstColors.dark,
      onBackground: ConstColors.light,
      surface: ConstColors.dark,
      onSurface: ConstColors.light,
    );
  }

  static AppBarTheme _getAppBarTheme(ColorScheme colorScheme) {
    return AppBarTheme(
      backgroundColor: Colors.transparent,
      foregroundColor: colorScheme.brightness == Brightness.dark
          ? ConstColors.light
          : ConstColors.dark,
      systemOverlayStyle: SystemUiOverlayStyle(
        statusBarIconBrightness: colorScheme.brightness == Brightness.dark
            ? Brightness.light
            : Brightness.dark,
        statusBarBrightness: colorScheme.brightness,
      ),
      elevation: 0,
      scrolledUnderElevation: 0,
    );
  }

  static ChipThemeData _getChipThemeData(ColorScheme colorScheme) {
    return ChipThemeData(
      shadowColor: Colors.transparent,
      backgroundColor: colorScheme.secondary,
      iconTheme: _getIconThemeData(colorScheme),
      side: BorderSide.none,
    );
  }

  static IconThemeData _getIconThemeData(ColorScheme colorScheme) {
    return IconThemeData(color: colorScheme.onPrimary);
  }

  static TextTheme _getTextTheme(ColorScheme colorScheme) {
    return TextTheme(
      headlineLarge: TextStyle(
        fontSize: 48,
        fontWeight: FontWeight.bold,
        color: colorScheme.onPrimary,
      ),
      headlineMedium: TextStyle(
        fontSize: 32,
        fontWeight: FontWeight.bold,
        color: colorScheme.onPrimary,
      ),
      labelMedium: TextStyle(
        fontSize: 14,
        fontWeight: FontWeight.w500,
        color: colorScheme.onPrimary,
      ),
      titleMedium: TextStyle(
        fontSize: 24,
        fontWeight: FontWeight.w600,
        color: colorScheme.onPrimary,
      ),
      titleSmall: TextStyle(
        fontSize: 16,
        fontWeight: FontWeight.bold,
        color: colorScheme.onPrimary,
      ),
    );
  }

  static FilledButtonThemeData _getFilledButtonThemeData(
    ColorScheme colorScheme,
  ) {
    return FilledButtonThemeData(
      style: ButtonStyle(
        backgroundColor: MaterialStateProperty.resolveWith<Color?>(
          (Set<MaterialState> states) {
            if (states.contains(MaterialState.pressed)) {
              return colorScheme.secondary;
            }
            return null;
          },
        ),
        shape: MaterialStateProperty.all<RoundedRectangleBorder>(
          RoundedRectangleBorder(
            borderRadius: BorderRadius.circular(8.0),
          ),
        ),
      ),
    );
  }
}
